"use strict";

/* exported logger */

var functions = require('../../Functions');
var logger = require('../../logger');

var AbstractSurfaceElement = require('./AbstractSurfaceElement');
var IdentifiedElement = require('../data/IdentifiedElement');

/**
 * Class representing overlay of the reaction on the map relevant for a specific
 * layout.
 *
 * @param layoutReaction
 *          {@link LayoutReaction} for which overlay is created
 * @param reactionData
 *          {@link Reaction data} of the reaction
 * @param map
 *          {@link AbstractCustomMap} where overlay should be placed
 * @param customized
 *          if true then coloring from layoutReaction is taken, if false then
 *          default is used. Second option is used when more than one layout is
 *          visualized (there is no simple way to visualize few layouts at the
 *          same time)
 */
function ReactionSurface(params) {
  // call super constructor
  AbstractSurfaceElement.call(this, params);

  var overlayData = params.layoutReaction;
  this.setReactionData(params.reaction);

  var color = params.color;
  this.width = 5.0;
  // list of google map objects associated with the reaction (like lines)

  if (overlayData !== undefined && overlayData !== null) {
    if (overlayData.width) {
      this.width = overlayData.width;
    }
  }
  if (color === undefined) {
    color = "#0000FF";
  }

  this.setColor(color);
  this.setCustomized(params.customized);
  this.setIdentifiedElement(new IdentifiedElement(this.getReactionData()));

  this.init();
}

ReactionSurface.prototype = Object.create(AbstractSurfaceElement.prototype);
ReactionSurface.prototype.constructor = ReactionSurface;

/**
 * Returns {@link google.maps.Bounds bounds} of all google maps elements
 * included in the object.
 *
 * @returns {@link google.maps.Bounds bounds} of all google maps elements
 *          included in the object
 */
ReactionSurface.prototype.getBounds = function () {
  var bounds = new google.maps.LatLngBounds();

  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    var line = this.getGoogleMapObjects()[i];
    for (var j = 0; j < line.getPath().getLength(); j++) {
      bounds.extend(line.getPath().getAt(j));
    }
  }
  return bounds;
};

ReactionSurface.prototype.setCustomized = function (customized) {
  this.customized = customized;
};

/**
 * Sets color to be used by the reaction overlay visualization when visualized
 * in customized mode.
 *
 * @param color
 *          new color value
 */
ReactionSurface.prototype.setColor = function (color) {
  this._color = color;
  var gmapObjects = this.getGoogleMapObjects();
  for (var i = 0; i < gmapObjects.length; i++) {
    gmapObjects[i].setOptions({
      strokeColor: color,
    });
  }
  this.customized = true;
};

/**
 * Sets width of line to be used by the reaction overlay visualization when
 * visualized in customized mode.
 *
 * @param width
 *          new width value
 */
ReactionSurface.prototype.setWidth = function (width) {
  if (width !== undefined || width !== null) {
    this.width = width;
  } else {
    this.width = 1.0;
  }

};

/**
 * Returns color that should be used when visualized in customized mode.
 *
 * @returns color that should be used when visualized in customized mode.
 */
ReactionSurface.prototype.getColor = function () {
  return this._color;
};

/**
 * Returns width that should be used when visualized in customized mode.
 *
 * @returns width that should be used when visualized in customized mode.
 */
ReactionSurface.prototype.getWidth = function () {
  return this.width;
};

/**
 * This function is used when some layouts are added/removed from visualization.
 * The syntax is compatible with AliasSurface class so they can be handled in
 * the same way. When visualizing more than one layout startX or endX will be
 * set to a value between 0..1 (not inclusive). In such case the visualization
 * of reaction should turn to some default mode where we know that reaction is
 * highlighted, but we have no more information. In this way user will know that
 * he should investigate the reaction manually.
 *
 * @param startX
 *          see {AliasSurface} class for details, in this implementation the
 *          only important information is that when there are more than one
 *          layout visualized at least one of the two parameters (startX,endX)
 *          will differ from {0.0, 1.0} values
 * @param endX
 *          see {AliasSurface} class for details, in this implementation the
 *          only important information is that when there are more than one
 *          layout visualized at least one of the two parameters (startX,endX)
 *          will differ from {0.0, 1.0} values
 */
ReactionSurface.prototype.setBoundsForAlias = function (startX, endX) {
  if (this.isCustomized() && (startX > 0 || endX < 1)) {
    this.changedToDefault();
  } else if (!this.isCustomized() && (startX <= 0 && endX >= 1)) {
    this.changedToCustomized();
  }
};

/**
 * Changes visualization of the ReactionSurface to default mode where we mark
 * reaction as highlighted, but we skip customized reaction layout data.
 */
ReactionSurface.prototype.changedToDefault = function () {
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    this.getGoogleMapObjects()[i].setOptions({
      strokeColor: "#0000FF",
      strokeWeight: 5,
    });
  }
  this.customized = false;
};

/**
 * Changes visualization of the ReactionSurface to customized mode where we mark
 * reaction as highlighted with customized reaction layout data.
 */
ReactionSurface.prototype.changedToCustomized = function() {
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    this.getGoogleMapObjects()[i].setOptions({
      strokeColor: this.getColor(),
      strokeWeight: this.getWidth(),
    });
  }
  this.customized = true;
};

/**
 * Creates {google.maps.Polyline} from input data.
 *
 * @param line
 *          raw data about line taken from server
 * @param color
 *          color that should be used for visualization of the line
 * @param width
 *          width that should be used for visualization of the line
 * @param map
 *          {google.maps.Map} where line should be visualized
 * @returns {google.maps.Polyline} from input data
 */
ReactionSurface.createLine = function (line, color, width, map) {
  var pointA = new google.maps.Point(line.start.x, line.start.y);
  var pointB = new google.maps.Point(line.end.x, line.end.y);
  var latLngA = map.fromPointToLatLng(pointA);
  var latLngB = map.fromPointToLatLng(pointB);
  var path = [latLngA, latLngB];

  var googleLine = new google.maps.Polyline({
    strokeColor: color,
    strokeOpacity: 1.0,
    strokeWeight: width,
    path: path,
    map: map.map
  });
  return googleLine;
};

/**
 * Returns {@link Reaction} data for this marker.
 *
 * @returns {@link Reaction} data for this marker
 */
ReactionSurface.prototype.getReactionData = function () {
  return this.reactionData;
};

ReactionSurface.prototype.getId = function () {
  return this._id;
};

ReactionSurface.prototype.setId = function (id) {
  this._id = id;
};

ReactionSurface.prototype.setReactionData = function (value) {
  if (value === undefined || value === null) {
    throw new Error("Reaction must be defined");
  }
  this.reactionData = value;
  this.setId(value.getId());
};

ReactionSurface.prototype.isCustomized = function () {
  return this.customized;
};

ReactionSurface.prototype.init = function () {
  var i;
  var line;
  var googleLine;
  for (i = 0; i < this.reactionData.startLines.length; i++) {
    line = this.reactionData.startLines[i];
    googleLine = ReactionSurface.createLine(line, this.getColor(), this.width, this.getCustomMap());
    this.addGoogleMapObject(googleLine);
  }
  for (i = 0; i < this.reactionData.endLines.length; i++) {
    line = this.reactionData.endLines[i];
    googleLine = ReactionSurface.createLine(line, this.getColor(), this.width, this.getCustomMap());
    this.addGoogleMapObject(googleLine);
  }
  for (i = 0; i < this.reactionData.midLines.length; i++) {
    line = this.reactionData.midLines[i];
    googleLine = ReactionSurface.createLine(line, this.getColor(), this.width, this.getCustomMap());
    this.addGoogleMapObject(googleLine);
  }
  if (!this.isCustomized()) {
    this.changedToDefault();
  }
};

ReactionSurface.create = function (params) {
  var overlayData = params.layoutReaction;

  var promise = Promise.resolve(params.color);
  if (overlayData !== undefined && overlayData !== null) {
    if (overlayData.width) {
      this.width = overlayData.width;
    }
    promise = functions.overlayToColor(overlayData);
  }
  return promise.then(function (color) {
    if (color === undefined) {
      color = "#0000FF";
    }
    params.color = color;
    return new ReactionSurface(params);
  });
};

module.exports = ReactionSurface;
